import { assocPath } from './utils';
export class BasePart {
    constructor(options, path) {
        this.options = options;
        this.path = path || null;
        if (this.options.isEmpty) {
            this.isEmpty = this.options.isEmpty;
        }
        if (this.options.isValid) {
            this.isValid = this.options.isValid;
        }
        if (this.options.required && ('defaultValue' in this.options)) {
            throw new Error('Wrong params! Conflict options "required" and defaultValue');
        }
    }
    process(data) {
        const path = this.getPath();
        const pathData = this.getDataByPath(data, path);
        let value = pathData == null ? pathData : this.getValue(pathData);
        const isEmpty = this.isEmpty(value);
        const isValid = this.isValid(value);
        const type = this.options.type.name || this.options.type.prototype.constructor.name;
        const pathTpl = path ? ` with path "${path}" ` : ' ';
        if (this.options.required) {
            if (isEmpty) {
                throw new Error(`Required field type "${type}"${pathTpl}is empty!`);
            }
        }
        if (('defaultValue' in this.options) && isEmpty) {
            value = this.options.defaultValue;
        }
        else {
            if (!isValid) {
                throw new Error(`Field${pathTpl}is invalid!`);
            }
        }
        return value;
    }
    getPath() {
        return this.options.path === null ? null : this.options.path || this.path;
    }
    isEmpty(data) {
        return data == null;
    }
    isValid(data) {
        return true;
    }
    getDataByPath(data, path) {
        if (this.options.parseValue) {
            if (path) {
                return this.options.parseValue(assocPath(path.split('.'), data));
            }
            else {
                return this.options.parseValue(data);
            }
        }
        else if (path != null) {
            return assocPath(path.split('.'), data);
        }
        else {
            return data;
        }
    }
}
//# sourceMappingURL=BasePart.js.map