export const assocPath = (path, store) => {
    return path.reduce((acc, item) => {
        return acc && acc.hasOwnProperty(item) ? acc[item] : null;
    }, store);
};
export const isObject = (some) => typeof some === 'object' && !Array.isArray(some) && some != null;
const numToLength = (length, num) => {
    let str = String(num);
    for (let i = str.length; i < length; i++) {
        str = '0' + str;
    }
    return str;
};
const dateParsers = [
    {
        pattern: 'YYYY',
        handler: (localDate) => String(localDate.getFullYear())
    },
    {
        pattern: 'YY',
        handler: (localDate) => String(localDate.getFullYear()).substr(2)
    },
    {
        pattern: 'MM',
        handler: (localDate) => String(numToLength(2, localDate.getMonth() + 1))
    },
    {
        pattern: 'M',
        handler: (localDate) => String(localDate.getMonth() + 1)
    },
    {
        pattern: 'DD',
        handler: (localDate) => String(numToLength(2, localDate.getDate()))
    },
    {
        pattern: 'D',
        handler: (localDate) => String(localDate.getDate())
    },
    {
        pattern: 'hh',
        handler: (localDate) => String(numToLength(2, localDate.getHours()))
    },
    {
        pattern: 'h',
        handler: (localDate) => String(localDate.getHours())
    },
    {
        pattern: 'mm',
        handler: (localDate) => String(numToLength(2, localDate.getMinutes()))
    },
    {
        pattern: 'm',
        handler: (localDate) => String(localDate.getMinutes())
    },
    {
        pattern: 'ss',
        handler: (localDate) => String(numToLength(2, localDate.getSeconds()))
    },
    {
        pattern: 's',
        handler: (localDate) => String(localDate.getSeconds())
    }
];
const isNumber = (num) => typeof num === 'number';
export function date(pattern, processor) {
    const localPatterns = [];
    let forFind = pattern;
    let parse;
    dateParsers.forEach((datePattern) => {
        if (forFind.indexOf(datePattern.pattern) !== -1) {
            forFind = forFind.replace(datePattern.pattern, '');
            localPatterns.push(datePattern);
        }
    });
    if (processor) {
        parse = (toParse) => {
            const result = processor(toParse);
            return isNumber(result) ? new Date(result) : result;
        };
    }
    else {
        parse = (data) => {
            return isNumber(data) ? new Date(data) : data;
        };
    }
    return (localDate) => {
        const _date = parse(localDate);
        return localPatterns.reduce((result, datePattern) => {
            return result.replace(datePattern.pattern, datePattern.handler(_date));
        }, pattern);
    };
}
//# sourceMappingURL=utils.js.map